import javax.swing.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observer;
import java.awt.*;

/**
 * Class to represent a 2d world that can hold turtles and
 * display them
 * 
 * Copyright Georgia Institute of Technology 2004
 * @author Barb Ericson ericson@cc.gatech.edu
 */
public class TurtleWorld extends JComponent implements ModelDisplay
{
  ////////////////// fields ///////////////////////
  
  /** the background color for the world */
  private Color background = Color.white;
  
  /** the width of the world */
  private int width = 640;
  
  /** the height of the world */
  private int height = 480;
  
  /** the list of turtles in the world */
  private List turtleList = new ArrayList();
  
  /** the JFrame to show this world in */
  private JFrame frame = new JFrame("Turtle World");
  
  ////////////////// the constructors ///////////////
  
  /**
   * Constructor that takes no arguments
   */
  public TurtleWorld()
  {
    // set the preferred size
    this.setPreferredSize(new Dimension(width,height));
    
    // add this panel to the frame
    frame.getContentPane().add(this);
    
    // pack the frame
    frame.pack();
    
    // show this world
    frame.setVisible(true);
    
  }
  
  ///////////////// methods ///////////////////////////
  
  /**
   * Method to paint this component
   * @param g the graphics context
   */
  public void paintComponent(Graphics g)
  {
    Turtle turtle = null;
    
    // set the color to the background color
    g.setColor(background);
    
    // clear the background
    g.fillRect(0,0,width,height);
    
    // loop drawing each turtle
    Iterator iterator = turtleList.iterator();
    while (iterator.hasNext())
    {
      turtle = (Turtle) iterator.next();
      turtle.paintComponent(g);
    }
      
  }
  
  /**
   * Metod to get the last turtle in this world 
   * @return the last turtle added to this world
   */
  public Turtle getLastTurtle()
  {
    return (Turtle) turtleList.get(turtleList.size() - 1);
  }
  
  
  /**
   * Method to add a model to this model displayer
   * @param model the model object to add
   */
  public void addModel(Object model)
  {
    turtleList.add((Turtle) model);
    repaint();
  }
  
  /**
   * Method to check if this world contains the passed
   * turtle
   * @return true if there else false
   */
  public boolean containsTurtle(Turtle turtle)
  {
    return (turtleList.contains(turtle));
  }
  
  /**
   * Method to get the width in pixels
   * @return the width in pixels
   */
  public int getWidth() { return width; }
  
  /**
   * Method to get the height in pixels
   * @return the height in pixels
   */
  public int getHeight() { return height; }
  
  /**
   * Method that allows the model to notify the display
   */
  public void modelChanged()
  {
    repaint();
  }
  
} // end of TurtleWorld class